@@ -278,4 +278,6 @@ napi_value init(napi_env env, napi_value exports) {
     return exports;
 }
 
-NAPI_MODULE(NODE_GYP_MODULE_NAME, init)
+NAPI_MODULE_INIT() {
+    return init(env, exports);
+}
