@@ -7,6 +7,7 @@ import (
 	"io/fs"
 	"io/ioutil"
 	"log"
+	"math"
 	"net/url"
 	"os"
 	"os/signal"
@@ -18,7 +19,6 @@ import (
 	"time"
 
 	"github.com/djherbis/atime"
-	humanize "github.com/dustin/go-humanize"
 	"github.com/matryer/try"
 	"github.com/tdewolff/argp"
 	min "github.com/tdewolff/minify/v2"
@@ -84,62 +84,74 @@ type Matches struct {
 	matches *[]string
 }
 
-func (scanner Matches) Scan(s []string) (int, error) {
+func (m Matches) Help() (string, string) {
+	val := ""
+	if 0 < len(*m.matches) {
+		val = fmt.Sprint(*m.matches)
+	}
+	return val, "[]string"
+}
+
+func (m Matches) Scan(name string, s []string) (int, error) {
 	n := 0
 	for _, item := range s {
 		if strings.HasPrefix(item, "-") {
 			break
 		}
-		*scanner.matches = append(*scanner.matches, item)
+		*m.matches = append(*m.matches, item)
 		n++
 	}
 	return n, nil
 }
 
-func (typenamer Matches) TypeName() string {
-	return "[]string"
-}
-
 type Includes struct {
 	filters *[]string
 }
 
-func (scanner Includes) Scan(s []string) (int, error) {
+func (m Includes) Help() (string, string) {
+	val := ""
+	if 0 < len(*m.filters) {
+		val = fmt.Sprint(*m.filters)
+	}
+	return val, "[]string"
+}
+
+func (m Includes) Scan(name string, s []string) (int, error) {
 	n := 0
 	for _, item := range s {
 		if strings.HasPrefix(item, "-") {
 			break
 		}
-		*scanner.filters = append(*scanner.filters, "+"+item)
+		*m.filters = append(*m.filters, "+"+item)
 		n++
 	}
 	return n, nil
 }
 
-func (typenamer Includes) TypeName() string {
-	return "[]string"
-}
-
 type Excludes struct {
 	filters *[]string
 }
 
-func (scanner Excludes) Scan(s []string) (int, error) {
+func (m Excludes) Help() (string, string) {
+	val := ""
+	if 0 < len(*m.filters) {
+		val = fmt.Sprint(*m.filters)
+	}
+	return val, "[]string"
+}
+
+func (m Excludes) Scan(name string, s []string) (int, error) {
 	n := 0
 	for _, item := range s {
 		if strings.HasPrefix(item, "-") {
 			break
 		}
-		*scanner.filters = append(*scanner.filters, "-"+item)
+		*m.filters = append(*m.filters, "-"+item)
 		n++
 	}
 	return n, nil
 }
 
-func (typenamer Excludes) TypeName() string {
-	return "[]string"
-}
-
 // Task is a minify task.
 type Task struct {
 	root string
@@ -165,6 +177,7 @@ var (
 	Error   *log.Logger
 	Warning *log.Logger
 	Info    *log.Logger
+	Debug   *log.Logger
 )
 
 func main() {
@@ -184,43 +197,49 @@ func run() int {
 	svgMinifier := svg.Minifier{}
 	xmlMinifier := xml.Minifier{}
 
+	preserve := []string{"mode", "timestamps"}
+	if supportsGetOwnership {
+		preserve = []string{"mode", "ownership", "timestamps"}
+	}
+
 	f := argp.New("minify")
 	f.AddRest(&inputs, "inputs", "Input files or directories, leave blank to use stdin")
-	f.AddOpt(&output, "o", "output", nil, "Output file or directory, leave blank to use stdout")
-	f.AddOpt(&oldmimetype, "", "mime", nil, "Mimetype (eg. text/css), optional for input filenames (DEPRECATED, use --type)")
-	f.AddOpt(&mimetype, "", "type", nil, "Filetype (eg. css or text/css), optional when specifying inputs")
-	f.AddOpt(Matches{&matches}, "", "match", nil, "Filename matching pattern, only matching filenames are processed")
-	f.AddOpt(Includes{&filters}, "", "include", nil, "Path inclusion pattern, includes paths previously excluded")
-	f.AddOpt(Excludes{&filters}, "", "exclude", nil, "Path exclusion pattern, excludes paths from being processed")
-	f.AddOpt(&extensions, "", "ext", nil, "Filename extension mapping to filetype (eg. css or text/css)")
-	f.AddOpt(&recursive, "r", "recursive", false, "Recursively minify directories")
-	f.AddOpt(&hidden, "a", "all", false, "Minify all files, including hidden files and files in hidden directories")
-	f.AddOpt(&list, "l", "list", false, "List all accepted filetypes")
-	f.AddOpt(&quiet, "q", "quiet", false, "Quiet mode to suppress all output")
-	f.AddOpt(argp.Count{&verbose}, "v", "verbose", nil, "Verbose mode, set twice for more verbosity")
-	f.AddOpt(&watch, "w", "watch", false, "Watch files and minify upon changes")
-	f.AddOpt(&sync, "s", "sync", false, "Copy all files to destination directory and minify when filetype matches")
-	f.AddOpt(&preserve, "p", "preserve", []string{"mode", "ownership", "timestamps"}, "Preserve options (mode, ownership, timestamps, links, all)")
-	f.AddOpt(&bundle, "b", "bundle", false, "Bundle files by concatenation into a single file")
-	f.AddOpt(&version, "", "version", false, "Version")
+	f.AddOpt(&output, "o", "output", "Output file or directory, leave blank to use stdout")
+	f.AddOpt(&oldmimetype, "", "mime", "Mimetype (eg. text/css), optional for input filenames (DEPRECATED, use --type)")
+	f.AddOpt(&mimetype, "", "type", "Filetype (eg. css or text/css), optional when specifying inputs")
+	f.AddOpt(Matches{&matches}, "", "match", "Filename matching pattern, only matching filenames are processed")
+	f.AddOpt(Includes{&filters}, "", "include", "Path inclusion pattern, includes paths previously excluded")
+	f.AddOpt(Excludes{&filters}, "", "exclude", "Path exclusion pattern, excludes paths from being processed")
+	f.AddOpt(&extensions, "", "ext", "Filename extension mapping to filetype (eg. css or text/css)")
+	f.AddOpt(&recursive, "r", "recursive", "Recursively minify directories")
+	f.AddOpt(&hidden, "a", "all", "Minify all files, including hidden files and files in hidden directories")
+	f.AddOpt(&list, "l", "list", "List all accepted filetypes")
+	f.AddOpt(&quiet, "q", "quiet", "Quiet mode to suppress all output")
+	f.AddOpt(argp.Count{&verbose}, "v", "verbose", "Verbose mode, set twice for more verbosity")
+	f.AddOpt(&watch, "w", "watch", "Watch files and minify upon changes")
+	f.AddOpt(&sync, "s", "sync", "Copy all files to destination directory and minify when filetype matches")
+	f.AddOpt(&preserve, "p", "preserve", "Preserve options (mode, ownership, timestamps, links, all)")
+	f.AddOpt(&bundle, "b", "bundle", "Bundle files by concatenation into a single file")
+	f.AddOpt(&version, "", "version", "Version")
 
-	f.AddOpt(&siteurl, "", "url", nil, "URL of file to enable URL minification")
-	f.AddOpt(&cssMinifier.Precision, "", "css-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
-	f.AddOpt(&htmlMinifier.KeepComments, "", "html-keep-comments", false, "Preserve all comments")
-	f.AddOpt(&htmlMinifier.KeepConditionalComments, "", "html-keep-conditional-comments", false, "Preserve all IE conditional comments")
-	f.AddOpt(&htmlMinifier.KeepDefaultAttrVals, "", "html-keep-default-attrvals", false, "Preserve default attribute values")
-	f.AddOpt(&htmlMinifier.KeepDocumentTags, "", "html-keep-document-tags", false, "Preserve html, head and body tags")
-	f.AddOpt(&htmlMinifier.KeepEndTags, "", "html-keep-end-tags", false, "Preserve all end tags")
-	f.AddOpt(&htmlMinifier.KeepWhitespace, "", "html-keep-whitespace", false, "Preserve whitespace characters but still collapse multiple into one")
-	f.AddOpt(&htmlMinifier.KeepQuotes, "", "html-keep-quotes", false, "Preserve quotes around attribute values")
-	f.AddOpt(&jsMinifier.Precision, "", "js-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
-	f.AddOpt(&jsMinifier.KeepVarNames, "", "js-keep-var-names", false, "Preserve original variable names")
-	f.AddOpt(&jsMinifier.Version, "", "js-version", 0, "ECMAScript version to toggle supported optimizations (e.g. 2019, 2020), by default 0 is the latest version")
-	f.AddOpt(&jsonMinifier.Precision, "", "json-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
-	f.AddOpt(&jsonMinifier.KeepNumbers, "", "json-keep-numbers", false, "Preserve original numbers instead of minifying them")
-	f.AddOpt(&svgMinifier.KeepComments, "", "svg-keep-comments", false, "Preserve all comments")
-	f.AddOpt(&svgMinifier.Precision, "", "svg-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
-	f.AddOpt(&xmlMinifier.KeepWhitespace, "", "xml-keep-whitespace", false, "Preserve whitespace characters but still collapse multiple into one")
+	f.AddOpt(&siteurl, "", "url", "URL of file to enable URL minification")
+	f.AddOpt(&cssMinifier.Precision, "", "css-precision", "Number of significant digits to preserve in numbers, 0 is all")
+	f.AddOpt(&htmlMinifier.KeepComments, "", "html-keep-comments", "Preserve all comments")
+	f.AddOpt(&htmlMinifier.KeepConditionalComments, "", "html-keep-conditional-comments", "Preserve all IE conditional comments (DEPRECATED)")
+	f.AddOpt(&htmlMinifier.KeepSpecialComments, "", "html-keep-special-comments", "Preserve all IE conditionals and SSI tags")
+	f.AddOpt(&htmlMinifier.KeepDefaultAttrVals, "", "html-keep-default-attrvals", "Preserve default attribute values")
+	f.AddOpt(&htmlMinifier.KeepDocumentTags, "", "html-keep-document-tags", "Preserve html, head and body tags")
+	f.AddOpt(&htmlMinifier.KeepEndTags, "", "html-keep-end-tags", "Preserve all end tags")
+	f.AddOpt(&htmlMinifier.KeepWhitespace, "", "html-keep-whitespace", "Preserve whitespace characters but still collapse multiple into one")
+	f.AddOpt(&htmlMinifier.KeepQuotes, "", "html-keep-quotes", "Preserve quotes around attribute values")
+	f.AddOpt(&jsMinifier.Precision, "", "js-precision", "Number of significant digits to preserve in numbers, 0 is all")
+	f.AddOpt(&jsMinifier.KeepVarNames, "", "js-keep-var-names", "Preserve original variable names")
+	f.AddOpt(&jsMinifier.Version, "", "js-version", "ECMAScript version to toggle supported optimizations (e.g. 2019, 2020), by default 0 is the latest version")
+	f.AddOpt(&jsonMinifier.Precision, "", "json-precision", "Number of significant digits to preserve in numbers, 0 is all")
+	f.AddOpt(&jsonMinifier.KeepNumbers, "", "json-keep-numbers", "Preserve original numbers instead of minifying them")
+	f.AddOpt(&svgMinifier.KeepComments, "", "svg-keep-comments", "Preserve all comments")
+	f.AddOpt(&svgMinifier.Precision, "", "svg-precision", "Number of significant digits to preserve in numbers, 0 is all")
+	f.AddOpt(&xmlMinifier.KeepWhitespace, "", "xml-keep-whitespace", "Preserve whitespace characters but still collapse multiple into one")
 	f.Parse()
 
 	if version {
@@ -265,6 +284,7 @@ func run() int {
 	Error = log.New(ioutil.Discard, "", 0)
 	Warning = log.New(ioutil.Discard, "", 0)
 	Info = log.New(ioutil.Discard, "", 0)
+	Debug = log.New(ioutil.Discard, "", 0)
 	if !quiet {
 		Error = log.New(os.Stderr, "ERROR: ", 0)
 		if 0 < verbose {
@@ -273,6 +293,9 @@ func run() int {
 		if 1 < verbose {
 			Info = log.New(os.Stderr, "INFO: ", 0)
 		}
+		if 2 < verbose {
+			Debug = log.New(os.Stderr, "DEBUG: ", 0)
+		}
 	}
 
 	// compile matches and regexps
@@ -304,7 +327,7 @@ func run() int {
 	if slash := strings.Index(mimetype, "/"); slash == -1 && 0 < len(mimetype) {
 		var ok bool
 		if mimetype, ok = extMap[mimetype]; !ok {
-			Error.Println("unknown filetype", mimetype)
+			Error.Printf("unknown filetype %v", mimetype)
 			return 1
 		}
 	}
@@ -337,34 +360,16 @@ func run() int {
 		return 1
 	}
 	if mimetype == "" {
-		if !recursive {
-			okAll := true
-			for _, input := range inputs {
-				ext := filepath.Ext(input)
-				if 0 < len(ext) {
-					ext = ext[1:]
-				}
-				if _, ok := extMap[ext]; !ok {
-					Error.Println("cannot infer mimetype from extension in", input, ", set --type explicitly")
-					okAll = false
-				}
-			}
-			if !okAll {
-				return 1
-			}
-		}
-		Info.Println("infer mimetype from file extensions")
+		Debug.Println("infer mimetype from file extensions")
 	} else {
-		Info.Println("use mimetype", mimetype)
+		Debug.Printf("use mimetype %v", mimetype)
 	}
-	if f.IsSet("preserve") {
-		if bundle {
-			Error.Println("--preserve cannot be used together with --bundle")
-			return 1
-		} else if useStdin || output == "" {
+	if 0 < len(preserve) && (useStdin || output == "") {
+		if f.IsSet("preserve") {
 			Error.Println("--preserve cannot be used together with stdin or stdout")
 			return 1
 		}
+		preserve = preserve[:0]
 	}
 	for _, option := range preserve {
 		switch option {
@@ -383,6 +388,9 @@ func run() int {
 			preserveLinks = true
 		}
 	}
+	if preserveOwnership && !supportsGetOwnership {
+		Warning.Println(fmt.Errorf("preserve ownership not supported on platform"))
+	}
 
 	////////////////
 
@@ -400,17 +408,14 @@ func run() int {
 	// set output file or directory, empty means stdout
 	dirDst := false
 	if output != "" {
-		dirDst = IsDir(output)
-		if !dirDst {
-			if 1 < len(inputs) && !bundle {
-				Error.Printf("stat %v: no such file or directory\n", output)
-				return 1
-			} else if len(inputs) == 1 {
-				if info, err := os.Lstat(inputs[0]); err == nil && !bundle && info.Mode().IsDir() && info.Mode()&os.ModeSymlink == 0 {
-					dirDst = true
-				}
-			}
+		if 0 < len(output) && output[len(output)-1] == os.PathSeparator {
+			dirDst = true
+		} else if !bundle && 1 < len(inputs) {
+			dirDst = true
+		} else if !bundle && len(inputs) == 1 && IsDir(inputs[0]) {
+			dirDst = true
 		}
+
 		if dirDst && bundle {
 			Error.Println("--bundle requires destination to be stdout or a file")
 			return 1
@@ -425,16 +430,16 @@ func run() int {
 		return 1
 	}
 	if output == "" {
-		Info.Println("minify to stdout")
+		Debug.Println("minify to stdout")
 	} else if !dirDst {
-		Info.Println("minify to output file", output)
+		Debug.Printf("minify to output file %v", output)
 	} else if output == "."+string(os.PathSeparator) {
-		Info.Println("minify to current working directory")
+		Debug.Println("minify to current working directory")
 	} else {
-		Info.Println("minify to output directory", output)
+		Debug.Printf("minify to output directory %v", output)
 	}
 	if useStdin {
-		Info.Println("minify from stdin")
+		Debug.Println("minify from stdin")
 	}
 
 	var tasks []Task
@@ -588,7 +593,7 @@ func run() int {
 	}
 
 	if !watch {
-		Info.Println("finished in", time.Since(start))
+		Info.Printf("finished in %v", time.Since(start))
 	}
 	if 0 < fails {
 		return 1
@@ -683,8 +688,7 @@ func createTasks(fsys fs.FS, inputs []string, output string) ([]Task, []string,
 		if preserveLinks && info.Mode()&os.ModeSymlink != 0 {
 			// copy symlink as is
 			if !sync {
-				Warning.Println("--sync not specified, omitting symbolic link", input)
-				continue
+				return nil, nil, fmt.Errorf("--sync not specified, omitting symbolic link %v", input)
 			}
 			task, err := NewTask(root, input, output, true)
 			if err != nil {
@@ -694,6 +698,15 @@ func createTasks(fsys fs.FS, inputs []string, output string) ([]Task, []string,
 		} else if info.Mode().IsRegular() {
 			valid := fileFilter(input) // don't filter mimetype
 			if valid || sync {
+				if !sync {
+					ext := filepath.Ext(input)
+					if 0 < len(ext) {
+						ext = ext[1:]
+					}
+					if _, ok := extMap[ext]; !ok {
+						return nil, nil, fmt.Errorf("cannot infer mimetype from extension in %v, set --type explicitly", input)
+					}
+				}
 				task, err := NewTask(root, input, output, !valid)
 				if err != nil {
 					return nil, nil, err
@@ -702,7 +715,7 @@ func createTasks(fsys fs.FS, inputs []string, output string) ([]Task, []string,
 			}
 		} else if info.Mode().IsDir() {
 			if !recursive {
-				Warning.Println("--recursive not specified, omitting directory", input)
+				Warning.Printf("--recursive not specified, omitting directory %v", input)
 				continue
 			}
 
@@ -734,7 +747,7 @@ func createTasks(fsys fs.FS, inputs []string, output string) ([]Task, []string,
 				if preserveLinks && d.Type()&os.ModeSymlink != 0 {
 					// copy symlink as is
 					if !sync {
-						Warning.Println("--sync not specified, omitting symbolic link", input)
+						Warning.Printf("--sync not specified, omitting symbolic link %v", input)
 						return nil
 					}
 					task, err := NewTask(root, input, output, true)
@@ -793,13 +806,13 @@ func minify(t Task) bool {
 			}
 			srcMimetype, ok := extMap[ext]
 			if !ok {
-				Warning.Println("cannot infer mimetype from extension in", src, ", set --type explicitly")
+				Warning.Printf("cannot infer mimetype from extension in %v, set --type explicitly", src)
 				return false
 			}
 			if fileMimetype == "" {
 				fileMimetype = srcMimetype
 			} else if srcMimetype != fileMimetype {
-				Warning.Println("inferred mimetype", srcMimetype, "of", src, "for concatenation unequal to previous mimetypes, set --type explicitly")
+				Warning.Printf("inferred mimetype %v of %v for concatenation unequal to previous mimetypes, set --type explicitly", srcMimetype, src)
 				return false
 			}
 		}
@@ -866,8 +879,8 @@ func minify(t Task) bool {
 			Error.Println(err)
 			return false
 		}
-		preserveAttributes(t.srcs[0], t.root, t.dst)
-		Info.Println("copy", srcName, "to", dstName)
+		preserveAttributes(t.srcs, t.root, t.dst)
+		Info.Printf("copy %v to %v", srcName, dstName)
 		return true
 	}
 
@@ -875,7 +888,7 @@ func minify(t Task) bool {
 	if err != nil {
 		fr.Close()
 		fw.Close()
-		Error.Println("cannot minify "+srcName+":", err)
+		Error.Printf("cannot minify %v: %v", srcName, err)
 		return false
 	}
 	w := bytes.NewBuffer(make([]byte, 0, len(b)))
@@ -884,7 +897,7 @@ func minify(t Task) bool {
 	startTime := time.Now()
 	if err = m.Minify(fileMimetype, w, bytes.NewReader(b)); err != nil {
 		w = bytes.NewBuffer(b) // copy original
-		Error.Println("cannot minify "+srcName+":", err)
+		Error.Printf("cannot minify %v: %v", srcName, err)
 		success = false
 	}
 
@@ -897,18 +910,18 @@ func minify(t Task) bool {
 		dur := time.Since(startTime)
 		speed := "Inf MB"
 		if 0 < dur {
-			speed = humanize.Bytes(uint64(float64(rLen) / dur.Seconds()))
+			speed = formatBytes(uint64(float64(rLen) / dur.Seconds()))
 		}
 		ratio := 1.0
 		if 0 < rLen {
 			ratio = float64(wLen) / float64(rLen)
 		}
 
-		stats := fmt.Sprintf("(%9v, %6v, %6v, %5.1f%%, %6v/s)", dur, humanize.Bytes(uint64(rLen)), humanize.Bytes(uint64(wLen)), ratio*100, speed)
+		stats := fmt.Sprintf("(%6v, %6v, %6v, %5.1f%%, %6v/s)", formatDuration(dur), formatBytes(uint64(rLen)), formatBytes(uint64(wLen)), ratio*100, speed)
 		if srcName != dstName {
-			fmt.Println(stats, "-", srcName, "to", dstName)
+			fmt.Printf("%v - %v to %v\n", stats, srcName, dstName)
 		} else {
-			fmt.Println(stats, "-", srcName)
+			fmt.Printf("%v - %v\n", stats, srcName)
 		}
 	}
 
@@ -933,47 +946,146 @@ func minify(t Task) bool {
 			break
 		}
 	}
-	preserveAttributes(t.srcs[0], t.root, t.dst)
+	preserveAttributes(t.srcs, t.root, t.dst)
 	return success
 }
 
-func preserveAttributes(src, root, dst string) {
-	if src == "" || dst == "" {
-		return
-	}
-Next:
-	srcInfo, err := os.Stat(src)
-	if err != nil {
-		Warning.Println(err)
+func preserveAttributes(srcs []string, root, dst string) {
+	if len(srcs) == 0 || dst == "" {
 		return
 	}
 
-	if preserveMode {
-		err = os.Chmod(dst, srcInfo.Mode().Perm())
-		if err != nil {
+	// only with --bundle we allow 1 < len(srcs)
+
+	// make sure we only set attributes on directories and files inside the root destination
+	var err error
+	for i := range srcs {
+		if srcs[i], err = filepath.Rel(root, srcs[i]); err != nil {
+			// should never occur
+			Error.Printf("src is not part of root path: src=%s root=%s", srcs[i], root)
+			return
+		}
+	}
+
+	srcInfos := make([]os.FileInfo, len(srcs))
+Next:
+	for i := range srcs {
+		if srcInfos[i], err = os.Stat(filepath.Join(root, srcs[i])); err != nil {
 			Warning.Println(err)
+			return
+		}
+	}
+
+	if preserveMode {
+		valid := true
+		perm := srcInfos[0].Mode().Perm()
+		for _, srcInfo := range srcInfos[1:] {
+			if srcInfo.Mode().Perm() != perm {
+				Warning.Println("src permissions are different, won't set dst permissions")
+				valid = false
+				break
+			}
+		}
+		if valid {
+			if err = os.Chmod(dst, perm); err != nil {
+				Warning.Println(err)
+			}
 		}
 	}
 	if preserveOwnership {
-		if uid, gid, ok := getOwnership(srcInfo); ok {
-			err = os.Chown(dst, uid, gid)
-			if err != nil {
-				Warning.Println(err)
+		if uid, gid, ok := getOwnership(srcInfos[0]); ok {
+			valid := true
+			for _, srcInfo := range srcInfos[1:] {
+				if uid2, gid2, ok2 := getOwnership(srcInfo); !ok2 || uid2 != uid || gid2 != gid {
+					Warning.Println("src ownership is different, won't set dst ownership")
+					valid = false
+					break
+				}
+			}
+			if valid {
+				if err = os.Chown(dst, uid, gid); err != nil {
+					Warning.Println(err)
+				}
 			}
 		} else {
-			Warning.Println(fmt.Errorf("preserve ownership not supported on platform"))
+			Warning.Println("unable to set file ownership")
 		}
 	}
 	if preserveTimestamps {
-		err = os.Chtimes(dst, atime.Get(srcInfo), srcInfo.ModTime())
-		if err != nil {
+		accessTime := atime.Get(srcInfos[0])
+		modTime := srcInfos[0].ModTime()
+		for _, srcInfo := range srcInfos[1:] {
+			if accessTime2 := atime.Get(srcInfo); accessTime2.After(accessTime) {
+				accessTime = accessTime2
+			}
+			if modTime2 := srcInfo.ModTime(); modTime2.After(modTime) {
+				modTime = modTime2
+			}
+		}
+		if err = os.Chtimes(dst, accessTime, modTime); err != nil {
 			Warning.Println(err)
 		}
 	}
-	if src != root {
-		// preserve on until root path
-		src = filepath.Dir(src)
-		dst = filepath.Dir(dst)
+
+	// when using --bundle there are no directories to preserve
+	if 1 < len(srcs) {
+		return
+	}
+
+	dst = filepath.Dir(dst)
+	if srcs[0] = filepath.Dir(srcs[0]); srcs[0] != "." {
+		// go up to but excluding the root path
 		goto Next
 	}
 }
+
+func formatBytes(size uint64) string {
+	if size < 10 {
+		return fmt.Sprintf("%d B", size)
+	}
+
+	units := []string{"B", "kB", "MB", "GB", "TB", "PB", "EB"}
+	scale := int(math.Floor((math.Log10(float64(size)) + math.Log10(2.0)) / 3.0))
+	value := float64(size) / math.Pow10(scale*3.0)
+	format := "%.0f %s"
+	if value < 10.0 {
+		format = "%.1f %s"
+	}
+	return fmt.Sprintf(format, value, units[scale])
+}
+
+func formatDuration(dur time.Duration) string {
+	str := ""
+	if 1.0 <= dur.Hours() {
+		hours, _ := math.Modf(dur.Hours())
+		str += fmt.Sprintf("%dh", int(hours+0.5))
+		dur -= time.Duration(int(hours+0.5) * 3600 * 1e9)
+	}
+	if 1.0 <= dur.Minutes() {
+		minutes, _ := math.Modf(dur.Minutes())
+		str += fmt.Sprintf("%dm", int(minutes+0.5))
+		dur -= time.Duration(int(minutes+0.5) * 60 * 1e9)
+	}
+	if 1.0 <= dur.Seconds() {
+		seconds, _ := math.Modf(dur.Seconds())
+		str += fmt.Sprintf("%ds", int(seconds+0.5))
+		dur -= time.Duration(int(seconds+0.5) * 1e9)
+	}
+	if 0 < len(str) {
+		return str
+	} else if dur == 0 {
+		return "0s"
+	} else if dur < 10 {
+		return fmt.Sprintf("%d ns", dur)
+	}
+
+	size := dur.Nanoseconds()
+	units := []string{"ns", "µs", "ms", "s"}
+	scale := int(math.Floor((math.Log10(float64(size)) + math.Log10(2.0)) / 3.0))
+	value := float64(size) / math.Pow10(scale*3.0)
+	format := "%.0f %s"
+	if value < 10.0 {
+		format = "%.1f %s"
+	}
+	return fmt.Sprintf(format, value, units[scale])
+}
