@@ -73,6 +73,9 @@ func TestCSS(t *testing.T) {
 		{"@import url(", `@import url(`},
 		{"@import url( ", `@import url( `},
 		{"@import url(  )", `@import ""`},
+
+		// bugs
+		{"a{@media screen and (min-width:1024px){ width: 40%; } & h1 { font-size: clamp(2.5rem, 1rem + 3vw, 3.5rem)}}", "a{@media screen and (min-width:1024px){width: 40%;}& h1 { font-size: clamp(2.5rem, 1rem + 3vw, 3.5rem)}}"}, // #602
 	}
 
 	m := minify.New()
@@ -410,6 +413,7 @@ func TestCSSInline(t *testing.T) {
 		{"color:rgba(255,255,255,0)", "color:rgba(255,255,255,0)"}, // #327
 		{"box-shadow:none", "box-shadow:none"},                     // #332
 		{"margin:var(--a,0px)", "margin:var(--a,0px)"},             // #371
+		{"background:transparent var(--bs-btn-close-bg) center/1em auto no-repeat", "background:var(--bs-btn-close-bg)50%/1em no-repeat"}, // #603
 
 		// go-fuzz
 		{"FONT-FAMILY: ru\"", "font-family:ru\""},
