@@ -146,7 +146,6 @@ func optimizeStmtList(list []js.IStmt, blockType blockType) []js.IStmt {
 	for i := 0; i < len(list); i++ { // read index
 		if ifStmt, ok := list[i].(*js.IfStmt); ok && !isEmptyStmt(ifStmt.Else) {
 			// if(!a)b;else c  =>  if(a)c; else b
-		IfElseLoop:
 			if unary, ok := ifStmt.Cond.(*js.UnaryExpr); ok && unary.Op == js.NotToken && isFlowStmt(lastStmt(ifStmt.Else)) {
 				ifStmt.Cond = unary.X
 				ifStmt.Body, ifStmt.Else = ifStmt.Else, ifStmt.Body
@@ -160,8 +159,6 @@ func optimizeStmtList(list []js.IStmt, blockType blockType) []js.IStmt {
 					list = append(list[:i+1], append([]js.IStmt{ifStmt.Else}, list[i+1:]...)...)
 				}
 				ifStmt.Else = nil
-			} else if ifStmt, ok = ifStmt.Else.(*js.IfStmt); ok && !isEmptyStmt(ifStmt.Else) {
-				goto IfElseLoop
 			}
 		}
 
@@ -192,13 +189,17 @@ func optimizeStmtList(list []js.IStmt, blockType blockType) []js.IStmt {
 					throwStmt.Value = commaExpr(left.Value, throwStmt.Value)
 					j--
 				} else if forStmt, ok := list[i].(*js.ForStmt); ok {
-					if varDecl, ok := forStmt.Init.(*js.VarDecl); ok && len(varDecl.List) == 0 || forStmt.Init == nil {
-						// TODO: only merge statements that don't have 'in' or 'of' keywords (slow to check?)
-						forStmt.Init = left.Value
-						j--
+					// TODO: only merge lhs expression that don't have 'in' or 'of' keywords (slow to check?)
+					if varDecl, ok := forStmt.Init.(*js.VarDecl); ok {
+						if len(varDecl.List) == 0 || forStmt.Init == nil {
+							forStmt.Init = left.Value
+							j--
+						} else if mergeVarDeclExprStmt(varDecl, left, true) {
+							j--
+						}
 					}
 				} else if whileStmt, ok := list[i].(*js.WhileStmt); ok {
-					// TODO: only merge statements that don't have 'in' or 'of' keywords (slow to check?)
+					// TODO: only merge lhs expression that don't have 'in' or 'of' keywords (slow to check?)
 					var body *js.BlockStmt
 					if blockStmt, ok := whileStmt.Body.(*js.BlockStmt); ok {
 						body = blockStmt
@@ -244,7 +245,7 @@ func optimizeStmtList(list []js.IStmt, blockType blockType) []js.IStmt {
 							j--
 						}
 					} else if forStmt, ok := list[i].(*js.ForStmt); ok {
-						// TODO: only merge statements that don't have 'in' or 'of' keywords (slow to check?)
+						// TODO: only merge lhs expression that don't have 'in' or 'of' keywords (slow to check?)
 						if forStmt.Init == nil {
 							forStmt.Init = left
 							j--
@@ -259,7 +260,7 @@ func optimizeStmtList(list []js.IStmt, blockType blockType) []js.IStmt {
 							j--
 						}
 					} else if whileStmt, ok := list[i].(*js.WhileStmt); ok {
-						// TODO: only merge statements that don't have 'in' or 'of' keywords (slow to check?)
+						// TODO: only merge lhs expression that don't have 'in' or 'of' keywords (slow to check?)
 						var body *js.BlockStmt
 						if blockStmt, ok := whileStmt.Body.(*js.BlockStmt); ok {
 							body = blockStmt
