@@ -32,7 +32,7 @@ func TestSVG(t *testing.T) {
 		// TODO: what about x="" y="" for viewBox?
 		//{`<svg width="24" height="24" viewBox="0 0 24 24"></svg>`, `<svg width="24" height="24"/>`},
 		{`<path x="a"> </path>`, `<path x="a"/>`},
-		{`<path x=""> </path>`, `<path/>`},
+		{`<path x=""> </path>`, `<path x=""/>`},
 		{`<path x=" a "/>`, `<path x="a"/>`},
 		{"<path x=\" a \n b \"/>", `<path x="a b"/>`},
 		{`<path x="5.0px" y="0%"/>`, `<path x="5" y="0"/>`},
@@ -52,9 +52,7 @@ func TestSVG(t *testing.T) {
 		{`<path fill="#fff"/>`, `<path fill="#fff"/>`},
 		{`<path fill="white"/>`, `<path fill="#fff"/>`},
 		{`<path fill="#ff0000"/>`, `<path fill="red"/>`},
-		{`<rect x="5" y="10" rx="2" ry="3">`, ``},
-		{`<rect x="5" y="10" height="40"/>`, ``},
-		{`<rect x="5" y="10" width="30" height="0%"/>`, ``},
+		{`<rect x="5" y="10" width="30" height="0%"/>`, `<rect x="5" y="10" width="30" height="0"/>`},
 		{`<rect x="5" y="10" width="30%" height="100%"/>`, `<rect x="5" y="10" width="30%" height="100%"/>`},
 		{`<svg contentStyleType="text/json ; charset=iso-8859-1"><style>{a : true}</style></svg>`, `<svg contentStyleType="text/json;charset=iso-8859-1"><style>{a : true}</style></svg>`},
 		{`<metadata><dc:title /></metadata>`, ``},
@@ -69,9 +67,10 @@ func TestSVG(t *testing.T) {
 
 		{`<polygon points="-0.1,"/>`, `<polygon points="-0.1,"/>`},                                   // #45
 		{`<path stroke="url(#UPPERCASE)"/>`, `<path stroke="url(#UPPERCASE)"/>`},                     // #117
-		{`<rect height="10"/><path/>`, `<path/>`},                                                    // #244
-		{`<rect height="10"><path/></rect>`, ``},                                                     // #244
+		{`<rect height="10"/><path/>`, `<rect height="10"/><path/>`},                                 // #244
+		{`<rect height="10"><path/></rect>`, `<rect height="10"><path/></rect>`},                     // #244
 		{`<foreignObject><div></div></foreignObject>`, `<foreignObject><div></div></foreignObject>`}, // #291
+		{`<svg x-foo=""/>`, `<svg x-foo=""/>`},                                                       // #576
 
 		// go fuzz
 		{`<0 d=09e9.6e-9e0`, `<0 d="09e9.6e-9e0"`},
